@@ -1,6 +1,22 @@
-/*--------------------------------------------------------------------------------
- * Welcome to the dark side of the moon...
- * MACROS
+/*****************************************************************************/
+/*  LibreDWG - free implementation of the DWG file format                    */
+/*                                                                           */
+/*  Copyright (C) 2009, 2010, 2018 Free Software Foundation, Inc.            */
+/*                                                                           */
+/*  This library is free software, licensed under the terms of the GNU       */
+/*  General Public License as published by the Free Software Foundation,     */
+/*  either version 3 of the License, or (at your option) any later version.  */
+/*  You should have received a copy of the GNU General Public License        */
+/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
+/*****************************************************************************/
+
+/*
+ * dec_macros.h: decoding macros (for encode, print, free in the C files)
+ * written by Felipe Castro
+ * modified by Felipe Corrêa da Silva Sances
+ * modified by Rodrigo Rodrigues da Silva
+ * modified by Reini Urban
+ * modified by Denis Pruchkovksy
  */
 
 #ifdef HAVE_CTYPE_H
@@ -171,7 +187,7 @@
 #define FIELD_MS(name,dxf)  FIELDG(name, MS, dxf)
 #define FIELD_TF(name,len,dxf) \
   { VECTOR_CHKCOUNT(name,TF,len) \
-   _obj->name = bit_read_TF(dat,(int)len); \
+    _obj->name = bit_read_TF(dat,(int)len); \
     LOG_INSANE( #name ": [%d TF " #dxf "]\n", len); \
     LOG_INSANE_TF(FIELD_VALUE(name), (int)len); }
 #define FIELD_TFF(name,len,dxf) \
@@ -383,18 +399,20 @@
 
 //check for overflow into next object (invalid num_elems)
 #define AVAIL_BITS() \
-  (obj ? (long)((obj->address + obj->size)*8 - bit_position(dat) + 20) \
-       : 0xff00L)
+  (obj ? (long long)((long long)(obj->address + obj->size)*8 - bit_position(dat) + 20) \
+       : 0xff00LL)
 #define TYPE_MAXELEMSIZE(type) (unsigned)dwg_bits_size[BITS_##type]
 #define VECTOR_CHKCOUNT(name,type,size) \
-  if ((long)((size)*TYPE_MAXELEMSIZE(type)) > AVAIL_BITS()) {           \
-    LOG_ERROR("Invalid " #name " vcount %ld. Need min. %u bits for %s, have %ld for %s.", \
-              (long)(size), (size)*TYPE_MAXELEMSIZE(type), #type, AVAIL_BITS(), obj?obj->dxfname:""); \
+  if ((long long)((size)*TYPE_MAXELEMSIZE(type)) > AVAIL_BITS()) { \
+    LOG_ERROR("Invalid " #name " size %lld. Need min. %u bits for " #type ", have %lld for %s.", \
+              (long long)(size), (unsigned)(size)*TYPE_MAXELEMSIZE(type), AVAIL_BITS(), \
+              obj && obj->dxfname ? obj->dxfname:""); \
     return DWG_ERR_VALUEOUTOFBOUNDS; }
 #define _VECTOR_CHKCOUNT(name,size,maxelemsize) \
-  if ((long)(size)*(maxelemsize) > AVAIL_BITS()) { \
-    LOG_ERROR("Invalid " #name " vcount %ld. Need min. %u bits, have %ld for %s.", \
-              (long)(size), (unsigned)(size)*(maxelemsize), AVAIL_BITS(), obj?obj->dxfname:""); \
+  if ((long long)(size)*(maxelemsize) > AVAIL_BITS()) { \
+    LOG_ERROR("Invalid " #name " size %lld. Need min. %u bits, have %lld for %s.", \
+              (long long)(size), (unsigned)(size)*(maxelemsize), AVAIL_BITS(), \
+              obj && obj->dxfname ? obj->dxfname:""); \
     size = 0; \
     return DWG_ERR_VALUEOUTOFBOUNDS; }
 #define HANDLE_VECTOR_CHKCOUNT(name,size) \
@@ -414,7 +432,7 @@
           _obj->name[vcount] = bit_read_##type(dat); \
           LOG_INSANE(#name "[%ld]: " FORMAT_##type "\n", \
                      (long)vcount, _obj->name[vcount]) \
-        } \
+        }\
     }
 #define FIELD_VECTOR_T(name, size, dxf) \
   if (_obj->size > 0) \
@@ -444,76 +462,105 @@
           _obj->name[vcount] = bit_read_##type(dat); \
           LOG_INSANE(#name "[%ld]: " FORMAT_##type " [" #type " %d]\n", \
                      (long)vcount, _obj->name[vcount], _dxf++) \
-        } \
+        }\
     }
 
 #define FIELD_VECTOR(name, type, size, dxf) FIELD_VECTOR_N(name, type, _obj->size, dxf)
 
 #define FIELD_2RD_VECTOR(name, size, dxf) \
   VECTOR_CHKCOUNT(name,2RD,_obj->size) \
-  _obj->name = (BITCODE_2RD *) calloc(_obj->size, sizeof(BITCODE_2RD)); \
-  for (vcount=0; vcount< (BITCODE_BL)_obj->size; vcount++)\
-    {\
-      FIELD_2RD(name[vcount], dxf); \
+  if (_obj->size > 0) \
+    { \
+      _obj->name = (BITCODE_2RD *) calloc(_obj->size, sizeof(BITCODE_2RD)); \
+      for (vcount=0; vcount< (BITCODE_BL)_obj->size; vcount++)\
+        {\
+          FIELD_2RD(name[vcount], dxf); \
+        }\
     }
 
 #define FIELD_2DD_VECTOR(name, size, dxf) \
   VECTOR_CHKCOUNT(name,2DD,_obj->size) \
-    _obj->name = (BITCODE_2RD *) calloc(_obj->size, sizeof(BITCODE_2RD)); \
-  FIELD_2RD(name[0], dxf); \
-  for (vcount = 1; vcount < (BITCODE_BL)_obj->size; vcount++)\
-    {\
-      FIELD_DD(name[vcount].x, FIELD_VALUE(name[vcount - 1].x), dxf); \
-      FIELD_DD(name[vcount].y, FIELD_VALUE(name[vcount - 1].x), dxf+10);\
-      LOG_TRACE(#name "[%ld]: (" FORMAT_BD ", " FORMAT_BD ") [DD %d]\n", \
-                (long)vcount, _obj->name[vcount].x, _obj->name[vcount].y, dxf) \
+  if (_obj->size > 0) \
+    { \
+      _obj->name = (BITCODE_2RD *) calloc(_obj->size, sizeof(BITCODE_2RD)); \
+      FIELD_2RD(name[0], dxf); \
+      for (vcount = 1; vcount < (BITCODE_BL)_obj->size; vcount++)\
+        {\
+          FIELD_DD(name[vcount].x, FIELD_VALUE(name[vcount - 1].x), dxf); \
+          FIELD_DD(name[vcount].y, FIELD_VALUE(name[vcount - 1].y), dxf+10);\
+          LOG_TRACE(#name "[%ld]: (" FORMAT_BD ", " FORMAT_BD ") [DD %d]\n", \
+                    (long)vcount, _obj->name[vcount].x, _obj->name[vcount].y, dxf) \
+        }\
     }
 
 #define FIELD_3DPOINT_VECTOR(name, size, dxf) \
   VECTOR_CHKCOUNT(name,3BD,_obj->size) \
-  _obj->name = (BITCODE_3DPOINT *) calloc(_obj->size, sizeof(BITCODE_3DPOINT)); \
-  for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++) \
-    {\
-      FIELD_3DPOINT(name[vcount], dxf); \
+  if (_obj->size > 0) \
+    { \
+      _obj->name = (BITCODE_3DPOINT *) calloc(_obj->size, sizeof(BITCODE_3DPOINT)); \
+      for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++) \
+        {\
+          FIELD_3DPOINT(name[vcount], dxf); \
+        }\
     }
 
 // shortest handle: 8 bit
 #define HANDLE_VECTOR_N(name, size, code, dxf) \
   VECTOR_CHKCOUNT(name,HANDLE,size) \
-    FIELD_VALUE(name) = (BITCODE_H*) calloc(size, sizeof(BITCODE_H)); \
-  for (vcount=0; vcount < (BITCODE_BL)size; vcount++) \
-    {\
-      FIELD_HANDLE_N(name[vcount], vcount, code, dxf);  \
+  if (size > 0) \
+    { \
+      FIELD_VALUE(name) = (BITCODE_H*) calloc(size, sizeof(BITCODE_H)); \
+      for (vcount=0; vcount < (BITCODE_BL)size; vcount++) \
+        {\
+          FIELD_HANDLE_N(name[vcount], vcount, code, dxf);  \
+        }\
     }
 
 #define HANDLE_VECTOR(name, sizefield, code, dxf) \
   HANDLE_VECTOR_N(name, FIELD_VALUE(sizefield), code, dxf)
 
-//skip non-zero bytes and a terminating zero
+//count 1 bytes, until non-1 bytes or a terminating zero
 #define FIELD_NUM_INSERTS(num_inserts, type, dxf) \
-      FIELD_VALUE(num_inserts)=0; \
-      while (bit_read_RC(dat)) \
+      FIELD_VALUE(num_inserts)=0; vcount=0; \
+      while ((vcount=(BITCODE_RC)bit_read_RC(dat))) \
         {\
+          if (vcount != 1) \
+            {\
+              LOG_ERROR("num_inserts [RC " FORMAT_RL "]: %d!\n", \
+                        FIELD_VALUE(num_inserts), (unsigned char)vcount) \
+              bit_advance_position(dat, -8); \
+              break;\
+            }\
           FIELD_VALUE(num_inserts)++;\
+          LOG_INSANE("num_inserts [RC " FORMAT_RL "]: %d\n", \
+                     FIELD_VALUE(num_inserts), (unsigned char)vcount)  \
         }\
       FIELD_G_TRACE(num_inserts, type, dxf)
 
 #define FIELD_XDATA(name, size)\
   _obj->name = dwg_decode_xdata(dat, _obj, _obj->size)
 
-#define REACTORS(code)\
-  obj->tio.object->reactors = calloc(obj->tio.object->num_reactors, sizeof(BITCODE_H)); \
-  for (vcount=0; vcount < obj->tio.object->num_reactors; vcount++) \
-    {\
-      VALUE_HANDLE_N(obj->tio.object->reactors[vcount], reactors, vcount, code, 330); \
+#define REACTORS(code) \
+  if (obj->tio.object->num_reactors > 0) \
+    { \
+      VECTOR_CHKCOUNT(reactors, HANDLE, obj->tio.object->num_reactors) \
+      obj->tio.object->reactors = calloc(obj->tio.object->num_reactors, sizeof(BITCODE_H)); \
+      for (vcount=0; vcount < obj->tio.object->num_reactors; vcount++) \
+        {\
+          VALUE_HANDLE_N(obj->tio.object->reactors[vcount], reactors, vcount, code, 330); \
+        } \
     }
 
-#define ENT_REACTORS(code)\
-  _ent->reactors = calloc(_ent->num_reactors, sizeof(BITCODE_H)); \
-  for (vcount=0; vcount < _ent->num_reactors; vcount++)\
-    {\
-      VALUE_HANDLE_N(_ent->reactors[vcount], reactors, vcount, code, 330); \
-    }
+#define ENT_REACTORS(code) \
+  if (_ent->num_reactors > 0) \
+    { \
+      VECTOR_CHKCOUNT(reactors, HANDLE, _ent->num_reactors) \
+      _ent->reactors = calloc(_ent->num_reactors, sizeof(BITCODE_H)); \
+      for (vcount=0; vcount < _ent->num_reactors; vcount++) \
+        { \
+          VALUE_HANDLE_N(_ent->reactors[vcount], reactors, vcount, code, 330); \
+        } \
+     }
 
 #define XDICOBJHANDLE(code)\
   SINCE(R_2004)\
@@ -579,9 +626,9 @@
     LOG_ERROR("Invalid " #name " in %s. No bytes left.\n", obj->dxfname); \
     return DWG_ERR_VALUEOUTOFBOUNDS; \
   } \
-  LOG_INSANE("REPEAT_CHKCOUNT %s." #name " x %ld: %ld > %ld?\n", \
-    obj->dxfname, (long)times, (long)((times)*sizeof(type)), AVAIL_BITS()); \
-  if (dat->version >= R_2004 && (long)((times)*sizeof(type)) > AVAIL_BITS()) { \
+  LOG_INSANE("REPEAT_CHKCOUNT %s." #name " x %ld: %lld > %lld?\n", \
+    obj->dxfname, (long)times, (long long)((times)*sizeof(type)), AVAIL_BITS()); \
+  if (dat->version >= R_2004 && (long long)((times)*sizeof(type)) > AVAIL_BITS()) { \
     LOG_ERROR("Invalid %s." #name " x %ld\n", obj->dxfname, (long)times); \
     return DWG_ERR_VALUEOUTOFBOUNDS; }
 #define REPEAT_CHKCOUNT_LVAL(name,times,type) \
@@ -590,29 +637,29 @@
     times = 0; \
     return DWG_ERR_VALUEOUTOFBOUNDS; \
   } \
-  LOG_INSANE("REPEAT_CHKCOUNT_LVAL %s." #name " x %ld: %ld > %ld?\n", \
-    obj->dxfname, (long)times, (long)((times)*sizeof(type)), AVAIL_BITS()); \
-  if (dat->version >= R_2004 && (long)((times)*sizeof(type)) > AVAIL_BITS()) { \
+  LOG_INSANE("REPEAT_CHKCOUNT_LVAL %s." #name " x %ld: %lld > %lld?\n", \
+    obj->dxfname, (long)times, (long long)((times)*sizeof(type)), AVAIL_BITS()); \
+  if (dat->version >= R_2004 && (long long)((times)*sizeof(type)) > AVAIL_BITS()) { \
     LOG_ERROR("Invalid %s." #name " x %ld\n", obj->dxfname, (long)times); \
     times = 0; \
     return DWG_ERR_VALUEOUTOFBOUNDS; }
 
 // unchecked with a constant
 #define REPEAT_CN(times, name, type) \
-  if (times) _obj->name = (type *) calloc(times, sizeof(type)); \
+  if (times > 0) _obj->name = (type *) calloc(times, sizeof(type)); \
   for (rcount1=0; rcount1<(BITCODE_BL)times; rcount1++)
 #define REPEAT_N(times, name, type) \
   REPEAT_CHKCOUNT(name,times,type) \
-  if (times) _obj->name = (type *) calloc(times, sizeof(type)); \
+  if (times > 0) _obj->name = (type *) calloc(times, sizeof(type)); \
   for (rcount1=0; rcount1<(BITCODE_BL)times; rcount1++)
 
 #define _REPEAT(times, name, type, idx) \
   REPEAT_CHKCOUNT_LVAL(name,_obj->times,type) \
-  if (_obj->times) _obj->name = (type *) calloc(_obj->times, sizeof(type)); \
+  if (_obj->times > 0) _obj->name = (type *) calloc(_obj->times, sizeof(type)); \
   for (rcount##idx=0; rcount##idx<(BITCODE_BL)_obj->times; rcount##idx++)
 #define _REPEAT_C(times, name, type, idx) \
   REPEAT_CHKCOUNT_LVAL(name,_obj->times,type) \
-  if (_obj->times) _obj->name = (type *) calloc(_obj->times, sizeof(type)); \
+  if (_obj->times > 0) _obj->name = (type *) calloc(_obj->times, sizeof(type)); \
   for (rcount##idx=0; rcount##idx<(BITCODE_BL)_obj->times; rcount##idx++)
 #define _REPEAT_N(times, name, type, idx) \
   if (_obj->name) \
@@ -659,23 +706,40 @@ EXPORT int dwg_add_##token (Dwg_Object *obj) \
   _ent->tio.token->parent = obj->tio.entity;\
   return 0; \
 } \
+\
+static int dwg_decode_##token##_private (Bit_Chain *dat, Bit_Chain *str_dat, \
+                                         Dwg_Object *restrict obj); \
+\
 /**Call dwg_add_##token and write the fields from the bitstream dat to the entity or object. */ \
 static int dwg_decode_##token (Bit_Chain *restrict dat, Dwg_Object *restrict obj) \
+{ \
+  Bit_Chain* str_dat; \
+  int error = dwg_add_##token(obj); \
+  if (error) \
+    return error; \
+  if (dat->version >= R_2007) { \
+    str_dat = calloc(1, sizeof(Bit_Chain)); /* separate string buffer */ \
+    if (!str_dat) return DWG_ERR_OUTOFMEM; \
+    *str_dat = *dat; \
+  } else { \
+    str_dat = dat; \
+  } \
+  error = dwg_decode_##token##_private (dat, str_dat, obj); \
+  if (dat->version >= R_2007) { \
+    free(str_dat); \
+  } \
+  return error; \
+} \
+\
+static int dwg_decode_##token##_private (Bit_Chain *dat, Bit_Chain *str_dat, \
+                                         Dwg_Object *restrict obj) \
 { \
   BITCODE_BL vcount, rcount1, rcount2, rcount3, rcount4; \
+  int error; \
   Dwg_Entity_##token *ent, *_obj; \
   Dwg_Object_Entity *_ent; \
   Dwg_Data* dwg = obj->parent; \
   Bit_Chain* hdl_dat = dat; \
-  Bit_Chain* str_dat; \
-  int error = dwg_add_##token(obj); \
-  if (error) return error; \
-  if (dat->version >= R_2007) { \
-    str_dat = calloc(1, sizeof(Bit_Chain)); /* seperate string buffer */ \
-    if (!str_dat) return DWG_ERR_OUTOFMEM; \
-    *str_dat = *dat; \
-  } else \
-    str_dat = dat; \
   LOG_INFO("Decode entity " #token " ")\
   _ent = obj->tio.entity; \
   ent = obj->tio.entity->tio.token;\
@@ -690,10 +754,10 @@ static int dwg_decode_##token (Bit_Chain *restrict dat, Dwg_Object *restrict obj
   } \
   if (error >= DWG_ERR_CRITICAL) return error;
 
+
 // Does size include the CRC?
 #define DWG_ENTITY_END \
   if (dat->version >= R_2007) { \
-    free(str_dat); \
     vcount  = (obj->size+obj->address)*8 - bit_position(hdl_dat); \
   } else { \
     vcount  = (obj->size+obj->address)*8 - bit_position(dat); \
@@ -706,7 +770,7 @@ static int dwg_decode_##token (Bit_Chain *restrict dat, Dwg_Object *restrict obj
 }
 
 #define DWG_OBJECT(token) \
-EXPORT int dwg_add_ ## token (Dwg_Object *obj) \
+EXPORT int dwg_add_##token (Dwg_Object *obj) \
 { \
   Dwg_Object_##token *_obj;\
   LOG_INFO("Add object " #token " ")\
@@ -722,23 +786,39 @@ EXPORT int dwg_add_ ## token (Dwg_Object *obj) \
   obj->tio.object->objid = obj->index; /* obj ptr itself might move */ \
   return 0; \
 } \
-static int dwg_decode_ ## token (Bit_Chain *restrict dat, Dwg_Object *restrict obj) \
+static int dwg_decode_##token##_private (Bit_Chain *dat, Bit_Chain *str_dat, \
+                                         Dwg_Object *restrict obj); \
+static int dwg_decode_##token (Bit_Chain *restrict dat, Dwg_Object *restrict obj) \
+{ \
+  Bit_Chain* str_dat; \
+  int error = dwg_add_##token(obj); \
+  if (error) \
+    return error; \
+  if (dat->version >= R_2007) { \
+    str_dat = calloc(1, sizeof(Bit_Chain)); /* separate string buffer */ \
+    if (!str_dat) return DWG_ERR_OUTOFMEM; \
+  } else { \
+    str_dat = dat; \
+  } \
+  error = dwg_decode_##token##_private (dat, str_dat, obj); \
+  if (dat->version >= R_2007) { \
+    free(str_dat); \
+  } \
+  return error; \
+} \
+\
+static int dwg_decode_##token##_private (Bit_Chain *dat, Bit_Chain *str_dat, \
+                                         Dwg_Object *restrict obj) \
 { \
   BITCODE_BL vcount, rcount1, rcount2, rcount3, rcount4; \
+  int error; \
   Dwg_Object_##token *_obj;\
   Dwg_Data* dwg = obj->parent;\
   Bit_Chain* hdl_dat = dat; /* handle stream initially the same */ \
-  Bit_Chain* str_dat; \
-  int error = dwg_add_##token(obj); \
-  if (error) return error; \
-  if (dat->version >= R_2007) { \
-    str_dat = calloc(1, sizeof(Bit_Chain)); /* seperate string buffer */ \
-    if (!str_dat) return DWG_ERR_OUTOFMEM; \
-  } else \
-    str_dat = dat; \
   LOG_INFO("Decode object " #token " ")\
   _obj = obj->tio.object->tio.token;\
-  error |= dwg_decode_object(dat, hdl_dat, str_dat, obj->tio.object); \
+  error = dwg_decode_object(dat, hdl_dat, str_dat, obj->tio.object); \
   if (error >= DWG_ERR_CRITICAL) return error;
 
+
 #define DWG_OBJECT_END DWG_ENTITY_END
