@@ -13,6 +13,7 @@
 /*
  * free.c: helper functions to free all spec fields
  * written by Reini Urban
+ * modified by Denis Pruchkovksy
  */
 
 #include "config.h"
@@ -66,9 +67,10 @@ static Bit_Chain *dat = &pdat;
 
 #define ANYCODE -1
 #define FIELD_HANDLE(name,code,dxf) VALUE_HANDLE(_obj->name,code,dxf)
-#define VALUE_HANDLE(ref,code,dxf) \
+#define VALUE_HANDLE(ref,_code,dxf) \
   if (ref) { \
-    if (!ref->obj && !ref->handleref.size && !ref->absolute_ref) free(ref); \
+    if (!(ref->handleref.size || (obj && ref->handleref.code > 5))) \
+      free(ref); ref = NULL; \
   } /* else freed globally */
 #define FIELD_DATAHANDLE(name,code,dxf) FIELD_HANDLE(name, code, dxf)
 #define FIELD_HANDLE_N(name,vcount,code,dxf) FIELD_HANDLE(name, code, dxf)
@@ -95,7 +97,7 @@ static Bit_Chain *dat = &pdat;
     }
 #define VALUE_TV(value,dxf) FREE_IF(value)
 #define VALUE_TF(value,dxf) FREE_IF(value)
-#define VALUE_TFF(value,dxf) 
+#define VALUE_TFF(value,dxf)
 #define FIELD_TU(name,dxf)  FIELD_TV(name,dxf)
 #define FIELD_TF(name,len,dxf) FIELD_TV(name,dxf)
 #define FIELD_TFF(name,len,dxf) {}
@@ -123,7 +125,7 @@ static Bit_Chain *dat = &pdat;
 // it all in the vector called 'name'.
 #define FIELD_VECTOR_N(name, type, size, dxf) \
   if (size && _obj->name) { \
-    for (vcount=0; vcount < (BITCODE_BL)size; vcount++)  \
+    for (vcount=0; vcount < (BITCODE_BL)size; vcount++) \
       FIELD_##type(name[vcount], dxf); \
   } \
   FIELD_TV(name,dxf);
@@ -157,7 +159,7 @@ static Bit_Chain *dat = &pdat;
   if (ent->reactors) { \
     for (vcount=0; vcount < ent->num_reactors; vcount++)\
       VALUE_HANDLE(ent->reactors[vcount], code, 330);  \
-    VALUE_TV(ent->reactors,0); \
+    VALUE_TV(ent->reactors, 0); \
   }
 #define XDICOBJHANDLE(code)\
   SINCE(R_2004)\
@@ -200,49 +202,76 @@ static int dwg_free_UNKNOWN_OBJ (Bit_Chain *restrict dat, Dwg_Object *restrict o
 
 #define DWG_ENTITY(token) \
 static int \
-dwg_free_ ##token (Bit_Chain *restrict _dat, Dwg_Object *restrict obj)\
-{\
+dwg_free_ ##token## _private (Bit_Chain *restrict _dat, Dwg_Object *restrict obj); \
+\
+static int \
+dwg_free_ ##token (Bit_Chain *restrict _dat, Dwg_Object *restrict obj) \
+{ \
+  int error;                \
+  LOG_HANDLE("Free entity " #token "\n") \
+  error = dwg_free_ ##token## _private(_dat, obj); \
+                            \
+  dwg_free_common_entity_data(obj); \
+  dwg_free_eed(obj);        \
+  FREE_IF(obj->tio.entity->tio.token); \
+  FREE_IF(obj->tio.entity); \
+  obj->parent = NULL;       \
+  return error;             \
+} \
+static int \
+dwg_free_ ##token## _private (Bit_Chain *restrict _dat, Dwg_Object *restrict obj) \
+{ \
   BITCODE_BL vcount, rcount1, rcount2, rcount3, rcount4;\
   Dwg_Entity_##token *ent, *_obj;\
-  Dwg_Object_Entity *_ent;\
+  Dwg_Object_Entity *_ent;  \
   Bit_Chain *hdl_dat = _dat;\
   Bit_Chain* str_dat = _dat;\
   Dwg_Data* dwg = obj->parent;\
-  int error = 0; \
-  LOG_HANDLE("Free entity " #token "\n")\
-  _ent = obj->tio.entity;\
+  int error = 0;            \
+  _ent = obj->tio.entity;   \
   _obj = ent = _ent->tio.token;
 
 #define DWG_ENTITY_END      \
-  dwg_free_common_entity_data(obj); \
-  dwg_free_eed(obj);        \
-  FREE_IF(_obj);            \
-  FREE_IF(obj->tio.entity); \
-  obj->parent = NULL;       \
   return 0;                 \
 }
 
 #define DWG_OBJECT(token) \
 static int \
+dwg_free_ ##token## _private (Bit_Chain *restrict _dat, Dwg_Object *restrict obj); \
+\
+static int \
 dwg_free_ ##token (Bit_Chain *restrict _dat, Dwg_Object *restrict obj) \
+{ \
+  int error;                                     \
+  Dwg_Object_##token *_obj;                      \
+  _obj = obj->tio.object->tio.token;             \
+  LOG_HANDLE("Free object " #token " %p\n", obj) \
+  if (strcmp(#token, "UNKNOWN_OBJ") && obj->supertype == DWG_SUPERTYPE_UNKNOWN) { \
+    _obj = NULL;                                 \
+    error = dwg_free_UNKNOWN_OBJ(_dat, obj);     \
+  } else {                                       \
+    error = dwg_free_ ##token## _private (_dat, obj); \
+  }                                              \
+  dwg_free_eed(obj);                             \
+  FREE_IF(_obj);                                 \
+  FREE_IF(obj->tio.object);                      \
+  obj->parent = NULL;                            \
+  return error;                                  \
+} \
+\
+static int \
+dwg_free_ ##token## _private (Bit_Chain *restrict _dat, Dwg_Object *restrict obj) \
 { \
   BITCODE_BL vcount, rcount1, rcount2, rcount3, rcount4; \
   Dwg_Object_##token *_obj;                      \
   Bit_Chain *hdl_dat = _dat;                     \
   Bit_Chain* str_dat = _dat;                     \
   Dwg_Data* dwg = obj->parent;                   \
-  int error = 0; \
-  LOG_HANDLE("Free object " #token " %p\n", obj) \
-  if (strcmp(#token, "UNKNOWN_OBJ") && obj->supertype == DWG_SUPERTYPE_UNKNOWN) \
-    return dwg_free_UNKNOWN_OBJ(_dat, obj); \
+  int error = 0;                                 \
   _obj = obj->tio.object->tio.token;
 
-/* obj itself is allocated via dwg->object[], dxfname is klass->dxfname */
+/* obj itself is allocated via dwg->object[], dxfname is klass->dxfname or static */
 #define DWG_OBJECT_END       \
-  dwg_free_eed(obj);         \
-  FREE_IF(_obj);             \
-  FREE_IF(obj->tio.object);  \
-  obj->parent = NULL;        \
   return 0;                  \
 }
 
@@ -288,27 +317,27 @@ dwg_free_xdata(Dwg_Object_XRECORD *obj, int size)
   obj->xdata = NULL;
 }
 
-static void
+EXPORT void
 dwg_free_eed(Dwg_Object* obj)
 {
   BITCODE_BL i;
-  if (obj->supertype == DWG_SUPERTYPE_OBJECT) {
-    Dwg_Object_Object* _obj = obj->tio.object;
-    for (i=0; i < _obj->num_eed; i++) {
-      if (_obj->eed[i].size)
-        FREE_IF(_obj->eed[i].raw);
-      FREE_IF(_obj->eed[i].data);
-    }
-    FREE_IF(_obj->eed);
-  }
-  else if (obj->supertype == DWG_SUPERTYPE_ENTITY) {
+  if (obj->supertype == DWG_SUPERTYPE_ENTITY) {
     Dwg_Object_Entity* _obj = obj->tio.entity;
     for (i=0; i < _obj->num_eed; i++) {
-      if (_obj->eed[i].size)
-        FREE_IF(_obj->eed[i].raw);
+      FREE_IF(_obj->eed[i].raw);
       FREE_IF(_obj->eed[i].data);
     }
     FREE_IF(_obj->eed);
+    _obj->num_eed = 0;
+  }
+  else {
+    Dwg_Object_Object* _obj = obj->tio.object;
+    for (i=0; i < _obj->num_eed; i++) {
+      FREE_IF(_obj->eed[i].raw);
+      FREE_IF(_obj->eed[i].data);
+    }
+    FREE_IF(_obj->eed);
+    _obj->num_eed = 0;
   }
 }
 
@@ -338,7 +367,7 @@ dwg_free_variable_type(Dwg_Data * dwg, Dwg_Object* obj)
 
   #undef WARN_UNHANDLED_CLASS
   #undef WARN_UNSTABLE_CLASS
-  
+
   return DWG_ERR_UNHANDLEDCLASS;
 }
 
@@ -605,9 +634,9 @@ dwg_free_object(Dwg_Object *obj)
     default:
       if (obj->type == obj->parent->layout_number)
         {
-          /*SINCE(R_13) {
-            dwg_free_LAYOUT(dat, obj); // avoid double-free, esp. in eed
-          }*/
+          SINCE(R_13) {
+            dwg_free_LAYOUT(dat, obj); // XXX avoid double-free, esp. in eed
+          }
         }
 
       else if ((error = dwg_free_variable_type(obj->parent, obj))
@@ -638,6 +667,7 @@ dwg_free_object(Dwg_Object *obj)
           else // not a class
             {
               free(obj->tio.unknown);
+              obj->tio.unknown = NULL;
             }
         }
     }
