@@ -80,6 +80,7 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
     fprintf(dat->fh, "%c%c", 0, 0); \
   }
 #endif
+#define VALUE_TFF(str,dxf)  VALUE_TV(str, dxf)
 #define VALUE_BINARY(value,size,dxf) \
 { \
   long len = size; \
@@ -101,9 +102,9 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
 //TODO
 #define VALUE_HANDLE(hdlptr, handle_code, dxf) \
   { \
-     uint32_t i = (uint32_t)hdlptr->absolute_ref; \
+     uint32_t _j = (uint32_t)hdlptr->absolute_ref; \
      GROUP(dxf); \
-     fwrite(&i, 4, 4, dat->fh); \
+     fwrite(&_j, sizeof(uint32_t), 1, dat->fh); \
   }
 #define FIELD_HANDLE(name, handle_code, dxf) VALUE_HANDLE(_obj->name, handle_code, dxf)
 
@@ -111,10 +112,10 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
     {                                \
     if (dat->version < R_14) {       \
       unsigned char icode = (unsigned char)code; \
-      fwrite(&icode, 1, 1, dat->fh); \
+      fwrite(&icode, sizeof(unsigned char), 1, dat->fh); \
     } else {                         \
       short icode = code;            \
-      fwrite(&icode, 2, 1, dat->fh); \
+      fwrite(&icode, sizeof(short), 1, dat->fh); \
     }                                \
   }
 #define FIELD_TV(name,dxf) \
@@ -133,6 +134,9 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
 #define HEADER_TV(name,dxf) \
     HEADER_9(name);\
     VALUE_TV(dwg->header_vars.name, dxf)
+#define HEADER_TU(name,dxf) \
+    HEADER_9(name);\
+    VALUE_TU(dwg->header_vars.name, dxf)
 #define HEADER_T(name,dxf) \
     HEADER_9(name);\
     VALUE_T(dwg->header_vars.name, dxf)
@@ -195,7 +199,7 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
   {\
     BITCODE_RC c = (value); \
     GROUP(dxf);\
-    fwrite(&c, 1, 1, dat->fh); \
+    fwrite(&c, sizeof(BITCODE_RC), 1, dat->fh); \
   }
 #define FIELD_RC(name,dxf) VALUE_RC(_obj->name, dxf)
 #define HEADER_RC(name,dxf) \
@@ -207,7 +211,7 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
   {\
     BITCODE_RS s = (value); \
     GROUP(dxf);\
-    fwrite(&s, 2, 1, dat->fh);\
+    fwrite(&s, sizeof(BITCODE_RS), 1, dat->fh);\
   }
 #define FIELD_RS(name,dxf) VALUE_RS(_obj->name, dxf)
 #define HEADER_RS(name,dxf) \
@@ -218,7 +222,7 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
   {\
     double d = (value);\
     GROUP(dxf);\
-    fwrite(&d, 1, 8, dat->fh); \
+    fwrite(&d, sizeof(double), 1, dat->fh); \
   }
 #define FIELD_RD(name,dxf) VALUE_RD(_obj->name,dxf)
 #define HEADER_RD(name,dxf) \
@@ -227,9 +231,9 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
 
 #define VALUE_RL(value,dxf)\
   {\
-    BITCODE_RL s = value;\
+    BITCODE_RL _s = value;\
     GROUP(dxf);\
-    fwrite(&s, 4, 1, dat->fh);\
+    fwrite(&_s, sizeof(BITCODE_RL), 1, dat->fh);\
   }
 #define FIELD_RL(name,dxf) VALUE_RL(_obj->name,dxf)
 #define HEADER_RL(name,dxf) \
@@ -272,9 +276,9 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
     GROUP(9);\
     fprintf (dat->fh, "$%s%c", #name, 0);\
     GROUP(dxf);\
-    fwrite(&s, 8, 1, dat->fh);\
+    fwrite(&s, sizeof(BITCODE_RLL), 1, dat->fh);\
   }
-  
+
 #define FIELD_MC(name,dxf) FIELD_RC(name,dxf)
 #define FIELD_MS(name,dxf)  FIELD_RS(name,dxf)
 #define FIELD_BT(name,dxf)    FIELD_BD(name, dxf);
@@ -293,7 +297,12 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
 #define FIELD_3BD(name,dxf) FIELD_3RD(name,dxf)
 #define FIELD_3BD_1(name,dxf) {FIELD_RD(name.x,dxf); FIELD_RD(name.y,dxf+1); FIELD_RD(name.z,dxf+2);}
 #define FIELD_3DPOINT(name,dxf) FIELD_3RD(name,dxf)
-#define FIELD_CMC(name,dxf) VALUE_RS(_obj->name.index, dxf)
+#define FIELD_CMC(color,dxf1,dxf2) { \
+  VALUE_RS(_obj->color.index, dxf1); \
+  if (dat->version >= R_2004 && dxf2 && _obj->color.rgb) { \
+    VALUE_RL(_obj->color.rgb, dxf2); \
+  } \
+}
 #define HEADER_CMC(name,dxf) \
     HEADER_9(name); VALUE_RS(dwg->header_vars.name.index, dxf)
 #define HEADER_TIMEBLL(name,dxf) \
@@ -305,16 +314,16 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
 // it all in the vector called 'name'.
 #define FIELD_VECTOR_N(name, type, size, dxf)\
   if (dxf) {\
-    for (vcount=0; vcount < (int)size; vcount++) \
+    for (vcount=0; vcount < (BITCODE_BL)size; vcount++) \
       VALUE (_obj->name[vcount], type, dxf); \
   }
 #define FIELD_VECTOR_T(name, size, dxf)\
   if (dxf) {\
     PRE (R_2007) { \
-      for (vcount=0; vcount < (int)_obj->size; vcount++) \
+      for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++) \
         VALUE_TV (_obj->name[vcount], dxf) \
     } else { \
-      for (vcount=0; vcount < (int)_obj->size; vcount++) \
+      for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++) \
         VALUE_TU (_obj->name[vcount], dxf) \
     }\
   }
@@ -324,7 +333,7 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
 
 #define FIELD_2RD_VECTOR(name, size, dxf)\
   if (dxf) {\
-    for (vcount=0; vcount < (int)_obj->size; vcount++)\
+    for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++)\
       {\
         FIELD_2RD(name[vcount], dxf);\
       }\
@@ -333,7 +342,7 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
 #define FIELD_2DD_VECTOR(name, size, dxf)\
   if (dxf) {\
     FIELD_2RD(name[0], 0);\
-    for (vcount = 1; vcount < (int)_obj->size; vcount++)\
+    for (vcount = 1; vcount < (BITCODE_BL)_obj->size; vcount++)\
       {\
         FIELD_2DD(name[vcount], FIELD_VALUE(name[vcount - 1].x), FIELD_VALUE(name[vcount - 1].y), dxf);\
       }\
@@ -341,7 +350,7 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
 
 #define FIELD_3DPOINT_VECTOR(name, size, dxf)\
   if (dxf) {\
-    for (vcount=0; vcount < (int)_obj->size; vcount++)\
+    for (vcount=0; vcount < (BITCODE_BL)_obj->size; vcount++)\
       {\
         FIELD_3DPOINT(name[vcount], dxf);\
       }\
@@ -349,7 +358,7 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
 
 #define HANDLE_VECTOR_N(name, size, code, dxf) \
   if (dxf) {\
-    for (vcount=0; vcount < (int)size; vcount++)\
+    for (vcount=0; vcount < (BITCODE_BL)size; vcount++)\
       {\
         FIELD_HANDLE_N(name[vcount], vcount, code, dxf);\
       }\
@@ -379,7 +388,7 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
       obj->tio.object->reactors) \
   { \
     VALUE_TV("{ACAD_REACTORS", 102) \
-    for (vcount=0; vcount < (int)obj->tio.object->num_reactors; vcount++)\
+    for (vcount=0; vcount < obj->tio.object->num_reactors; vcount++)\
       {\
         VALUE_HANDLE(obj->tio.object->reactors[vcount], code, 330);\
       }\
@@ -388,7 +397,7 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
 #define ENT_REACTORS(code)\
   if (dat->version >= R_13 && _obj->num_reactors && _obj->reactors) {\
     VALUE_TV("{ACAD_REACTORS", 102) \
-    for (vcount=0; vcount < (int)_obj->num_reactors; vcount++)\
+    for (vcount=0; vcount < _obj->num_reactors; vcount++)\
       {\
         VALUE_HANDLE(_obj->reactors[vcount], code, 330);\
       }\
@@ -413,18 +422,27 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat,
 #define END_STRING_STREAM
 #define START_HANDLE_STREAM
 
+#ifndef DEBUG_CLASSES
+static int
+dwg_dxfb_TABLECONTENT (Bit_Chain *restrict dat, const Dwg_Object *restrict obj)
+{
+  (void)dat; (void)obj;
+  return 0;
+}
+#else
 static int
 dwg_dxfb_TABLECONTENT (Bit_Chain *restrict dat, const Dwg_Object *restrict obj);
+#endif
 
 #define DWG_ENTITY(token) \
 static int \
 dwg_dxfb_##token (Bit_Chain *restrict dat, const Dwg_Object *restrict obj) \
 {\
-  int error = 0; \
-  long vcount, rcount1, rcount2, rcount3, rcount4; \
+  BITCODE_BL vcount, rcount1, rcount2, rcount3, rcount4; \
   Dwg_Data* dwg = obj->parent; \
   Dwg_Entity_##token *ent, *_obj;\
   Dwg_Object_Entity *_ent;\
+  int error = 0; \
   if (!strcmp(#token, "GEOPOSITIONMARKER"))\
     RECORD(POSITIONMARKER)\
   else if (dat->version < R_13 && strlen(#token) == 10 && !strcmp(#token, "LWPOLYLINE")) \
@@ -451,7 +469,7 @@ dwg_dxfb_##token (Bit_Chain *restrict dat, const Dwg_Object *restrict obj) \
       obj->handle.size,\
       obj->handle.value) \
     GROUP(330); \
-    fwrite(&i, 4, 4, dat->fh); \
+    fwrite(&i, sizeof(uint32_t), 1, dat->fh); \
   } \
   SINCE(R_13) { \
     VALUE_HANDLE_NAME (obj->parent->header_vars.BLOCK_RECORD_MSPACE, 330, BLOCK_HEADER); \
@@ -464,10 +482,11 @@ dwg_dxfb_##token (Bit_Chain *restrict dat, const Dwg_Object *restrict obj) \
 static int \
 dwg_dxfb_ ##token (Bit_Chain *restrict dat, const Dwg_Object *restrict obj) \
 { \
-  long vcount, rcount1, rcount2, rcount3, rcount4;\
+  BITCODE_BL vcount, rcount1, rcount2, rcount3, rcount4;\
   Bit_Chain *hdl_dat = dat;\
   Dwg_Data* dwg = obj->parent;\
   Dwg_Object_##token *_obj;\
+  int error = 0; \
   LOG_INFO("Object " #token ":\n")\
   _obj = obj->tio.object->tio.token;\
   if (!dwg_obj_is_control(obj)) { \
@@ -480,11 +499,11 @@ dwg_dxfb_ ##token (Bit_Chain *restrict dat, const Dwg_Object *restrict obj) \
     else if (obj->type != DWG_TYPE_BLOCK_HEADER) \
       RECORD(token)                              \
     SINCE(R_13) { \
-      uint32_t i = (uint32_t)obj->handle.value; \
+      uint32_t _i = (uint32_t)obj->handle.value; \
       int dxf = 5; \
       if (obj->type == DWG_TYPE_DIMSTYLE) dxf = 105; \
       GROUP(dxf);\
-      fwrite(&i, 4, 4, dat->fh); \
+      fwrite(&_i, sizeof(uint32_t), 1, dat->fh); \
       _XDICOBJHANDLE(3); \
       _REACTORS(4); \
     } \
@@ -594,9 +613,9 @@ dxfb_cvt_tablerecord(Bit_Chain *restrict dat, const Dwg_Object *restrict obj,
 // 5 written here first
 #define COMMON_TABLE_CONTROL_FLAGS \
   SINCE(R_13) { \
-    uint32_t i = (uint32_t)ctrl->handle.value; \
+    uint32_t _i = (uint32_t)ctrl->handle.value; \
     GROUP(5); \
-    fwrite(&i, 4, 4, dat->fh); \
+    fwrite(&_i, sizeof(uint32_t), 1, dat->fh); \
   } \
   SINCE(R_14) \
     VALUE_H (_ctrl->null_handle, 330); \
@@ -756,6 +775,7 @@ dwg_dxfb_object(Bit_Chain *restrict dat, const Dwg_Object *restrict obj)
       return dwg_dxfb_MLINE(dat, obj);
 #else
       LOG_WARN("Unhandled Entity MLINE in out_dxfb %u/%lX", obj->index, obj->handle.value)
+      if (0) dwg_dxfb_MLINE(dat, obj);
       return DWG_ERR_UNHANDLEDCLASS;
 #endif
     case DWG_TYPE_BLOCK_CONTROL:
@@ -844,8 +864,8 @@ dxfb_common_entity_handle_data(Bit_Chain *restrict dat, const Dwg_Object *restri
   Dwg_Object_Entity *ent;
   //Dwg_Data *dwg = obj->parent;
   Dwg_Object_Entity *_obj;
-  int i, error = 0;
-  long unsigned int vcount = 0;
+  BITCODE_BL vcount = 0;
+  int error = 0;
   ent = obj->tio.entity;
   _obj = ent;
 
@@ -1076,8 +1096,6 @@ dxfb_tables_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
         }
       // avoid unused warnings
       dwg_dxfb_PROXY_ENTITY(dat, &dwg->object[0]);
-      dwg_dxfb_UNKNOWN_ENT(dat, &dwg->object[0]);
-      dwg_dxfb_UNKNOWN_OBJ(dat, &dwg->object[0]);
       ENDTAB();
     }
   SINCE (R_13)
@@ -1105,12 +1123,12 @@ dxfb_tables_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
     }
     for (i=0; i<dwg->block_control.num_entries; i++)
       {
-        Dwg_Object *obj = dwg_ref_object(dwg, dwg->block_control.block_headers[i]);
-        if (obj && obj->type == DWG_TYPE_BLOCK_HEADER &&
-            obj != mspace && obj != pspace)
+        Dwg_Object *_o = dwg_ref_object(dwg, dwg->block_control.block_headers[i]);
+        if (_o && _o->type == DWG_TYPE_BLOCK_HEADER &&
+            _o != mspace && _o != pspace)
           {
             RECORD(BLOCK_RECORD);
-            error |= dwg_dxfb_BLOCK_HEADER(dat, obj);
+            error |= dwg_dxfb_BLOCK_HEADER(dat, _o);
           }
       }
     ENDTAB();
@@ -1124,7 +1142,6 @@ static int
 dxfb_blocks_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
 {
   int error = 0;
-  unsigned int i;
   Dwg_Object_BLOCK_CONTROL *_ctrl = &dwg->block_control;
   /* let's see if this control block is correct... */
   Dwg_Object_Ref *msref = dwg->header_vars.BLOCK_RECORD_MSPACE;
@@ -1162,11 +1179,13 @@ dxfb_blocks_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
 
   if (psref && psref->obj && psref->obj->tio.object->tio.BLOCK_HEADER->block_entity)
     hdr = psref->obj;
-  else
+  else if (_ctrl->paper_space)
     hdr = _ctrl->paper_space->obj;
+  else
+    hdr = NULL;
 
   if (hdr) {
-      Dwg_Object *obj = get_first_owned_block(hdr);
+      obj = get_first_owned_block(hdr);
       while (obj)
         {
           error |= dwg_dxfb_object(dat, obj);
@@ -1186,7 +1205,7 @@ static int
 dxfb_entities_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
 {
   int error = 0;
-  long unsigned int i;
+  BITCODE_BL i;
 
   SECTION(ENTITIES);
   for (i=0; i<dwg->num_objects; i++)
@@ -1204,7 +1223,7 @@ static int
 dxfb_objects_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
 {
   int error = 0;
-  long unsigned int i;
+  BITCODE_BL i;
 
   SECTION(OBJECTS);
   for (i=0; i<dwg->num_objects; i++)
