@@ -27,6 +27,7 @@
 #define IF_IS_ENCODER 0
 #define IF_IS_DECODER 0
 #define IF_IS_DXF 0
+#define IF_IS_FREE 0
 
 #ifndef ACTION
 # error ACTION define missing: decode, encode, dxf, ...
@@ -41,8 +42,8 @@
 #ifndef M_PI_2
 # define M_PI_2      1.57079632679489661923132169163975144
 #endif
-#define rad2deg(ang) (ang)*180.0/M_PI_2
-#define deg2rad(ang) (ang) ? M_PI_2/((ang)*180.0) : 0.0
+#define rad2deg(ang) (ang)*90.0/M_PI_2
+#define deg2rad(ang) (ang) ? M_PI_2/((ang)*90.0) : 0.0
 
 #ifndef VALUE_HANDLE
 # define VALUE_HANDLE(value, handle_code, dxf)
@@ -50,6 +51,12 @@
 #ifndef VALUE_TV
 # define VALUE_TV(value, dxf)
 #endif
+#ifndef VALUE_TF
+# define VALUE_TF(value, dxf)
+#endif
+#ifndef VALUE_TFF
+# define VALUE_TFF(value, dxf)
+#endif
 #ifndef VALUE_3BD
 # define VALUE_3BD(value, dxf)
 #endif
@@ -57,8 +64,12 @@
 # define VALUE_BL(value, dxf)
 #endif
 // logging format overrides
-#ifndef FIELD_BLh
-# define FIELD_BLh(name, dxf) FIELD_BL(name, dxf)
+#ifndef FIELD_RLx
+# define FIELD_RLx(name, dxf) FIELD_RL(name, dxf)
+#endif
+#ifndef FIELD_BLx
+# define FIELD_BLx(name, dxf) FIELD_BL(name, dxf)
+# define FIELD_BSx(name, dxf) FIELD_BS(name, dxf)
 #endif
 #ifndef FIELD_RCu
 # define FIELD_RCu(name, dxf) FIELD_RC(name, dxf)
@@ -80,6 +91,13 @@
   LOG_TRACE(#name ": (" FORMAT_BD ", " FORMAT_BD ", " FORMAT_BD ") [" #type " %d]\n", \
             _obj->name.x, _obj->name.y, _obj->name.z, dxf)
 #endif
+#ifndef LOG_TRACE_TF
+#define LOG_TRACE_TF(var,len)
+#define LOG_INSANE_TF(var,len)
+#endif
+#ifndef FIELD_EMC
+#define FIELD_EMC(a,b,c) FIELD_CMC(a,b,c)
+#endif
 
 #ifdef IS_ENCODER
 #undef ENCODER
@@ -126,6 +144,8 @@
 #if defined(IS_FREE)
 #undef FREE
 #define FREE    if (1)
+#undef IF_IS_FREE
+#define IF_IS_FREE 1
 #else
 #define END_REPEAT(field)
 #endif
@@ -133,6 +153,10 @@
 #define R11OPTS(b) _ent->opts_r11 & b
 #define R11FLAG(b) _ent->flag_r11 & b
 
+#define DECODE_UNKNOWN_BITS \
+  DECODER { dwg_decode_unknown(dat, (Dwg_Object *restrict)obj); } \
+  FREE { VALUE_TF(obj->unknown_bits, 0); }
+
 #ifndef COMMON_TABLE_FLAGS
 #define COMMON_TABLE_FLAGS(owner, acdbname) \
   PRE (R_13) \
@@ -195,25 +219,32 @@
 #endif
 
 
+#ifndef FIELD_VECTOR_N1
+# define FIELD_VECTOR_N1(name, type, size, dxf) FIELD_VECTOR_N(name, type, size, dxf)
+#endif
+
 #ifndef REPEAT
 
 #define REPEAT_CN(times, name, type) \
   if (_obj->name) \
-    for (rcount1=0; rcount1<(long)times; rcount1++)
+    for (rcount1=0; rcount1<(BITCODE_BL)times; rcount1++)
 #define REPEAT_N(times, name, type) \
   if (dat->version >= R_2000 && times > 0x1000) { \
     fprintf(stderr, "Invalid rcount1 %ld", (long)times); return DWG_ERR_VALUEOUTOFBOUNDS; } \
   if (_obj->name) \
-    for (rcount1=0; rcount1<(long)times; rcount1++)
+    for (rcount1=0; rcount1<(BITCODE_BL)times; rcount1++)
 
 #define _REPEAT(times, name, type, idx) \
   if (dat->version >= R_2000 && _obj->times > 0x1000) { \
     fprintf(stderr, "Invalid rcount " #idx " %ld", (long)_obj->times); return DWG_ERR_VALUEOUTOFBOUNDS; } \
   if (_obj->name) \
-    for (rcount##idx=0; rcount##idx<(long)_obj->times; rcount##idx++)
+    for (rcount##idx=0; rcount##idx<(BITCODE_BL)_obj->times; rcount##idx++)
 #define _REPEAT_C(times, name, type, idx) \
   if (_obj->name) \
-    for (rcount##idx=0; rcount##idx<(long)_obj->times; rcount##idx++)
+    for (rcount##idx=0; rcount##idx<(BITCODE_BL)_obj->times; rcount##idx++)
+#define _REPEAT_N(times, name, type, idx) \
+  if (_obj->name) \
+    for (rcount##idx=0; rcount##idx<(BITCODE_BL)times; rcount##idx++)
 #define REPEAT(times, name, type)  _REPEAT(times, name, type, 1)
 #define REPEAT2(times, name, type) _REPEAT(times, name, type, 2)
 #define REPEAT3(times, name, type) _REPEAT(times, name, type, 3)
@@ -225,5 +256,4 @@
 
 #endif
 
-
 #endif /* SPEC_H */
